package org.zeroturnaround.javarebel.integration.google;

import java.lang.reflect.Method;
import org.zeroturnaround.javarebel.LoggerFactory;

/**
 * @author Rein Raudjärv
 */
public class ThreadUtil {
  
  /**
   * Thread.getStackTrace() method which was added in Java 5.
   */
  private static final Method GET_STACK_TRACE_METHOD;
  
  /*
   * This class should only be referenced from code
   * that is executed under Java 5+ (but was compiled under Java 4).
   */
  static {
    Method m = null;
    try {
      m = Thread.class.getDeclaredMethod("getStackTrace", new Class[0]);
    } catch (Throwable e) {
      LoggerFactory.getInstance().error(e);
    }
    GET_STACK_TRACE_METHOD = m;
  }
  
  /**
   * Returns stack trace of the current Thread.
   * 
   * This method may be called only on Java 5+.
   */
  public static StackTraceElement[] getStackTrace() throws Exception {
    return getStackTrace(Thread.currentThread());
  }
  
  /**
   * Returns stack trace of the given Thread.
   * 
   * This method may be called only on Java 5+.
   */
  public static StackTraceElement[] getStackTrace(Thread t) throws Exception {
    if (GET_STACK_TRACE_METHOD == null)
      return null;
    
    StackTraceElement[] result = (StackTraceElement[]) GET_STACK_TRACE_METHOD.invoke(t, new Object[0]);
    return result;
  }

}
